import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:flutter_quill/flutter_quill.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:youtube_player_flutter/youtube_player_flutter.dart' as yti;
import 'package:youtube_player_iframe/youtube_player_iframe.dart';

class WebYoutubeVideoApp extends StatefulWidget {
  const WebYoutubeVideoApp(
      {required this.videoUrl, required this.context, required this.readOnly});

  final String videoUrl;
  final BuildContext context;
  final bool readOnly;

  @override
  _WebYoutubeVideoAppState createState() => _WebYoutubeVideoAppState();
}

class _WebYoutubeVideoAppState extends State<WebYoutubeVideoApp> {
  YoutubePlayerController? _controller;

  @override
  void initState() {
    super.initState();
    final videoId = yti.YoutubePlayer.convertUrlToId(widget.videoUrl);
    _controller = YoutubePlayerController(
      initialVideoId: 'K18cpp_-gP8',
      params: YoutubePlayerParams(
        showControls: true,
        showFullscreenButton: true,
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final defaultStyles = DefaultStyles.getInstance(context);
    if (_controller == null) {
      if (widget.readOnly) {
        return RichText(
          text: TextSpan(
              text: widget.videoUrl,
              style: defaultStyles.link,
              recognizer: TapGestureRecognizer()
                ..onTap = () => launch(widget.videoUrl)),
        );
      }

      return RichText(
          text: TextSpan(text: widget.videoUrl, style: defaultStyles.link));
    }

    return Container(
      height: 300,
      child: YoutubePlayerControllerProvider(
        controller: _controller!,
        child: YoutubePlayerIFrame(
          aspectRatio: 16 / 9,
        ),
      ),
    );
  }
}
